/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2009 The LS-Universe Team

    Homepage: http://www.ls-universe.info

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#if !defined(AFX_TRAY_H__4E26031B_8BE2_47C2_803A_FEBFB4262B31__INCLUDED_)
#define AFX_TRAY_H__4E26031B_8BE2_47C2_803A_FEBFB4262B31__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "TrayButton.h"
#include <algorithm>

class GhostTray;
class GhostButton;

//Alpha Transparency
#ifndef WS_EX_LAYERED
    #define WS_EX_LAYERED           0x00080000L
    #define LWA_COLORKEY            0x00000001L
    #define ULW_COLORKEY            0x00000001L
    #define LWA_ALPHA               0x00000002L
    #define ULW_ALPHA               0x00000002L
    #define ULW_OPAQUE              0x00000003L
#endif // WS_EX_LAYERED

#ifndef AC_SRC_ALPHA
    #define AC_SRC_ALPHA            0x00000001L
#endif

#ifndef WM_MOUSEWHEEL
    #define WM_MOUSEWHEEL 0x020A
#endif

#ifndef NIIF_USER
    #define NIIF_USER       0x00000004
#endif

typedef vector<TrayButton *> TrayButtonList;
typedef vector<TrayButton *>::iterator TrayButtonListIterator;

typedef vector<GhostTray *>GhostTrayList;
typedef vector<GhostTray *>::iterator GhostTrayListIterator;

typedef vector<GhostButton *>GhostButtonList;
typedef vector<GhostButton *>::iterator GhostButtonListIterator;

struct INFOTIPUPDATE_CONFIG
{
    int x;
    int y;
    TrayButton *button;
};

typedef vector<INFOTIPUPDATE_CONFIG *>BalloonTooltipList;
typedef vector<INFOTIPUPDATE_CONFIG *>::iterator BalloonTooltipListIterator;

class Tray  
{

public:

    Tray(const string &name);
    ~Tray();

    friend class TrayButton;

    void Load(HINSTANCE hInstance, HWND setbox = 0);
    void Reconfigure(bool refresh = false);

    const HWND &GethWnd() const { return hWnd; }
    const string &GetName() const { return strName; }
    const HWND &GetBox() const { return box; }
    bool IsAlwaysOnTop() const { return bAlwaysOnTop; }
    bool IsVisible() const { return bVisible; }
    bool IsStartHidden() const { return bStartHidden; }
    bool IsAlphaMap() const { return bAlphaMap; }
    bool IsShowAllowed() const { return (bHideIfEmpty && iBackupExportedButtonCount == 0 ? true : false); }

    void Reposition(int type, int newx, int newy, int newwidth, int newheight, int steps = 0, int time = 0, bool repositionBy = false);

    void SetAlwaysOnTop(bool bNewAlwaysOnTop, bool bFirstTime = false);
    void SetAlpha(int newAlphaTransparency);
    void SetAlphaFade(int newAlphaTransparency, int step, int delay);

    void Repaint();
    void UpdateRegion();

    void Show();    
    void Hide();

    void ToggleHiddenIcons();

    void CreateLayout(bool clearBackground = true, int width = -1, int height = -1);

    void ScrollIcons(bool next, int amount);

    int GetMaxIconSize() { return iMaxIconSize; };

    vector<CLASS_CONFIG> HideList;
    vector<CLASS_CONFIG> NoDragList;
    vector<CLASS_CONFIG> SortList;

	vector<NOTIFYICONDATA> delayedSetVersion;

    WNDPROC wpOrigToolTipProc;  

    //----------------------------------------------------
    // InfoTips
    //----------------------------------------------------

    BalloonTooltipList listBalloonTips;
    INFOTIPUPDATE_CONFIG UpdateConfig;

    void HideInfoTip();

private:

    //-----------------------------------------------
    // Essential Settings
    //-----------------------------------------------

    string strName;

    HWND hWnd;
    HINSTANCE hInstance;
    HWND box;

    int actualx;
    int actualy;
    int actualwidth;
    int actualheight;

    bool bTrueTransparency;

    bool bStartHidden;

    bool bAlwaysOnTop;

    bool bNormalWindowzOrder;

    string strBehindWindow;
    HWND hWndBehindWindow;

    //-----------------------------------------------
    // Settings
    //-----------------------------------------------

    bool bHideIfEmpty;

    bool bBorderDrag;

    bool bNoDragnDrop;

    bool bFocusOnEnter;

    bool bLoopingTrayIcons;

    int iCleanUpInterval;

    bool bFireClicksAlways;

    //-----------------------------------------------
    // Internal Switches and Settings
    //-----------------------------------------------

    bool bVisible;
    bool bAutoHidden;

    bool bMouseInside;

    bool bSkipButton;

    bool bParentDestroyNotificationReceived;

    bool bSkipBackup;

    bool bInCriticalProcess;

    //For correct adding and removing
    int iBackupExportedButtonCount;

    int iStartWithPosition;

    int iLines_RowsAdded;

    bool bHiddenVisible;

    //-----------------------------------------------
    // Lists (Overlays)
    //-----------------------------------------------

    GhostTrayList listGhostTrays;
    GhostButtonList listGhostButtons;

    TrayButtonList listTrayButtons;
    TrayButtonList listDisplayedTrayButtons;

    bool bHasOverlayConditional;

    //-----------------------------------------------
    // Painting/Layout
    //-----------------------------------------------

    HBITMAP oldbackupBMP;
    HDC backupDC;
    HBITMAP backupBitmap;

    void Create_xPaintClassSupport();
    void Destroy_xPaintClassSupport();

    PaintTexture *pTrayTexture;

    PaintIcon *pNormalIcon;
    PaintIcon *pHoverIcon;
    PaintIcon *pHiddenIcon;

    int iLeftBorder;
    int iTopBorder;
    int iRightBorder;
    int iBottomBorder;
    
    int iLayout;
    int iDirection;
    int iWrapDirection;
    int iWrapCount;

    int iLines_Rows;
    int iMaxLines_Rows;

    //General Button Settings
    int iXSpacing;
    int iYSpacing;

    int iMaxIconSize;

    //AutoSize
    bool bAutoSize;
    int iAutoSizeSteps;
    int iAutoSizeDelay;

    void AutoSizeFunction(int icons, bool bSkipAnimation = false);

    //-----------------------------------------------
    // Alpha
    //-----------------------------------------------

    bool bAlphaMap;
    bool bAlphaFade;
    int iAlphaTransparency;

    int iUpdateAlpha;

    int iAlphaFadeStep;
    int iAlphaFadeDelay;
    int iAlphaFadeBangEndValue;

    bool bAlphaInitialized;
    bool bLayeredFlag;

    //-----------------------------------------------
    // Temporary Help Pointers
    //-----------------------------------------------

    //For ReposButtons
    TrayButton *pMoveCapture;
    //For Tooltips
    TrayButton *pCapture;

    TrayButton *pChangeState;

    TrayButton *pDontPaintAddedTrayIcon;

    int iStartupBlocker;

    //----------------------------------------------------
    // Actions
    //----------------------------------------------------

    string trayLButtonDown;
    string trayLButtonUp;
    string trayRButtonDown;
    string trayRButtonUp;
    string trayMButtonDown;
    string trayMButtonUp;
    string trayMWheelDown;
    string trayMWheelUp;
    string trayEnter;
    string trayLeave;
    string trayAdd;
    string trayRemove;
    string trayMove;
    string trayResize;

    string trayShow;
    string trayHide;

    string trayLineAdd;
    string trayLineRemove;

    string trayOnIconHide;
    string trayOnIconUnHide;

    //----------------------------------------------------
    // TrayIcon Management
    //----------------------------------------------------

    int IconAdd(NOTIFYICONDATA *pnid);
    int IconModify(NOTIFYICONDATA *pnid);
    int IconDelete(NOTIFYICONDATA *pnid);

    //----------------------------------------------------
    // Sorting
    //----------------------------------------------------

    void SortTrayIcons(vector<TrayButton *> &traybuttons)
    {
        sort( traybuttons.begin(), traybuttons.end(), CompareTrayIcons );
    }

    static bool CompareTrayIcons(TrayButton *b1, TrayButton *b2)
    {
        if ( b1->bUserHidden == b2->bUserHidden )
        {
            if ( b1->iFixedPositionInTray < b2->iFixedPositionInTray )
                return true;
            else
                return false;
        }
        else
        {
            if ( !b1->bUserHidden && b2->bUserHidden )
                return true;
            else
                return false;
        }
    };

    //----------------------------------------------------
    // ToolTips
    //----------------------------------------------------

    PaintTooltip *pTrayTooltip;
    HWND hWndToolTip;

    bool bShowToolTip;

    //----------------------------------------------------
    // InfoTips
    //----------------------------------------------------

    HWND hWndInfoTip;

    bool bShowInfoTips;
    bool bInfoTipsEnabled;
    string strOnInfoTip;
    
    COLORREF clrInfoTipColor;
    COLORREF clrInfoTipTextColor;

    bool bAutoHideTimerIsRunning;

    bool CreateInfoTipWindow();

    void ShowInfoTip(TrayButton *button, int x, int y);

    void DestroyInfoTip();

private:

    void UpdateWindowTransparency(int newAlphaTransparency);

    TrayButton *ButtonAtPoint( int x, int y );

    int CheckDisplayedTrayIcons();

    void AmountOfTrayIconsChanged(bool bSkipAnimation = false);

protected:

    virtual TrayButton* onLButtonDblClk(int x, int y);
    virtual TrayButton* onLButtonDown(int x, int y, LPARAM lParam);
    virtual TrayButton* onLButtonUp(int x, int y);
    virtual TrayButton* onMButtonDblClk(int x, int y);
    virtual TrayButton* onMButtonDown(int x, int y);
    virtual TrayButton* onMButtonUp(int x, int y);
    virtual TrayButton* onRButtonDblClk(int x, int y);
    virtual TrayButton* onRButtonDown(int x, int y);
    virtual TrayButton* onRButtonUp(int x, int y); 
    virtual void onWheelDown(int x, int y);
    virtual void onWheelUp(int x, int y);
    virtual void onMouseEnter();
    virtual void onMouseLeave();
    virtual TrayButton* onMouseMove(int x, int y);
    virtual void onPaint(HDC hDC);
    virtual void onWindowPosChanging(WINDOWPOS *windowPos);
    virtual void onMove(int x, int y);
    virtual void onSize(int width, int height);
    virtual void onTimer(int timerID);

private:

    BOOL onWindowMessage( UINT message, WPARAM wParam, LPARAM lParam, LRESULT &lResult );

public:

    static LRESULT WINAPI WindowProcedure( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam );

};

#endif // !defined(AFX_TRAY_H__4E26031B_8BE2_47C2_803A_FEBFB4262B31__INCLUDED_)
